/**************************************************************
 *
 * File: hw2a_soln.c
 * Author: Steve Wolfman
 *
 * Description: Sample solution to CSE142 01sp project #2A.
 * See http://www.cs.washington.edu/142/ (navigate to hw#2, 01sp) for
 * description of assignment. 
 *
 **************************************************************/

#include <stdio.h>   /* For I/O. */
#include "debug.h"   /* For debug_printf. */
#include <stdlib.h>  /* For rand/srand. */
#include <time.h>    /* For time (to seed rand). */
#include <assert.h>  /* For assert. */

/********* Useful constants **************/
#define UNRANDOM_SEED 0    /* 0 means seed as randomly as possible.  if
                              this has a non-zero value, that value is
                              used as the random seed, giving consistent 
                              choices of random #s from run to run. */

/********* Function prototypes ***********/
int    play_game(char mode);                  /* Play the LMAD game once.  */
int    random_int(int range);                 /* Produce a random integer. */

/********* Main function *****************/
int main(void)
{
  char dummy;            /* Dummy var for pause at end. */
  
  play_game('i');        /* Play one game..
                          * For now, ignores return type 
                          * and hardwires parameter. */

  printf("Type any key and then hit ENTER to continue.. ");
  scanf(" %c", &dummy);
  
  /* Report success. */
  return 0;
}

/*********************************************************************
 * play_game:                                        
 *   takes a mode.                         
 *   in Part A, ignore this parameter.               
 *   in Part B:   ('s' for auto-switch,              
 *                 'k' for auto-keep)      
 *   and plays a game of LMAD (our style). In 's' or 'k' mode, the
 *   program makes all decisions. In switch, it always switches
 *   doors. In keep, it never does. In interactive (Part A) mode, the
 *   user makes all decisions.  
 *   Returns a 1 for a win or 0 for a loss.
 *                                                   
 * Part A preconditions: none.                       
 * Part B preconditions: mode is one of {'s', 'k'}   
 * postcondition: returns 1 iff won                  
 *********************************************************************/
int play_game(char mode)
{
  int cable_door;       /* The number of the winning door. */
  int pizza_door1,
      pizza_door2;      /* The two losing doors. */
  int door_choice;
  int revealed_door,    /* The door Monty reveals. */
      switch_door;      /* The remaining door (you can switch to). */
  int result;           /* Win/loss result. */
  int scanf_result;     /* Storage for checking scanf calls.   */
  char switch_choice;   /* Whether the user wants to switch. */
  
  /* Check preconditions. */
  /* assert(mode == 's' || mode == 'k'); */ /* Not useful until part B. */

  debug_printf("\n----------------------------------------------\n");
  debug_printf("Begining trial with mode %c.\n", mode);
  
  /****************************
   * Set up the game          *
   ****************************/
  cable_door = random_int(3) + 1;
  pizza_door1 = (cable_door) % 3 + 1;
  pizza_door2 = (cable_door + 1) % 3 + 1;

  debug_printf("Pssst... the cable is behind door #%d!\n", cable_door);
  debug_printf("Doors #%d and %d are just pizza.\n", pizza_door1, pizza_door2);

  /*****************************************
   * Make the initial door choice          *
   *****************************************/

  printf("Please select a door: 1, 2, or 3: ");
  scanf_result = scanf("%d", &door_choice);

  if (scanf_result != 1 || door_choice < 1 || door_choice > 3) {
    door_choice = 1;
    printf("Improper selection for door. Using %d.\n", door_choice);
  }
  
  /****************************
   * Reveal a door            *
   ****************************/

  /* Depending on whether door choice matches winning door, reveal
     appropriate door. */
  if (door_choice == cable_door) {
    /* Reveal one of the other two randomly. */
    debug_printf("You chose wisely. I'll reveal a random door.\n");
    
    if (random_int(2)) {
      revealed_door = pizza_door1;
    }
    else {
      revealed_door = pizza_door2;
    }
  }
  /* In either remaining case, must reveal the unchosen pizza. */
  else if (door_choice == pizza_door1) {
    debug_printf("You chose poorly. I know what door to reveal.\n");
    revealed_door = pizza_door2;
  }
  else {
    debug_printf("You chose poorly. I know what door to reveal.\n");
    revealed_door = pizza_door1;
  }

  /* The remaining door must be the choice to switch to. 
     Play an ugly mathematical trick: the leftover door will be
     the sum of the doors minus the two eliminated doors. */
  switch_door = (1 + 2 + 3) - revealed_door - door_choice;
  
  /* Reveal a door. */
  printf("Monty tells you there's pizza behind door #%d.\n",
         revealed_door);
  debug_printf("Who knows what's behind door #%d?\n", switch_door);
  
  /****************************
   * Choose whether to switch *
   ****************************/

  printf("Do you wish to switch or keep? (s/k) ");
  scanf_result = scanf(" %c", &switch_choice);
  if (scanf_result != 1 || 
      (switch_choice != 's' && switch_choice != 'k')) {
    switch_choice = 'k';
    printf("Improper selection for switching. Using %c.\n", switch_choice);
  }

  if (switch_choice == 's') {
    debug_printf("I can't TAKE IT! I switch!\n");
    door_choice = switch_door;
  }

  debug_printf("New choice is door #%d.\n", door_choice);

  /****************************
   * Find the result.         *
   ****************************/
  result = (door_choice == cable_door);

  if (result) {
    printf("You won!\n");
  }
  else {
    printf("You lost.\n");
  }

  return 0;
}


/**********************************************************************
 * random_int                                        
 *   takes a positive number defining the range over which to draw a
 *   random number; returns random integer from 0 to num - 1: [0, num)
 *
 * Examples:
 *   int i = random_int(3) + 1;   (i's value is 1, 2, or 3 at random)
 *   int j = random_int(2);       (j is either 0 or 1. This is handy
 *                                 because this means 50% of the time,
 *                                 j is true and 50% of the time it's 
 *                                 false)
 *                                                   
 * precondition: range > 0                           
 * postcondition: returns an int from 0 to range-1   
 **********************************************************************/
int random_int(int range)
{
  static int initialized = 0;  /* Makes sure we only seed once. */
  double random_num;           /* Stores a random on [0, 1)     */
  int result;                  /* Stores our final result.      */
  
  /* Check preconditions. */
  assert(range > 0);
  
  /* "Seed" the random number generator exactly once. */
  if (!initialized) {
    if (UNRANDOM_SEED) {
      srand(UNRANDOM_SEED);
    }
    else {
      srand(time(NULL));
    }
    /* First "random" value is not very random. Discard it. */
    random_num = (double)rand();
    initialized = 1;
  }
  
  /* Get a random number in the range [0, 1) */
  random_num = (double)rand() / ((double)RAND_MAX + 1);
  
  /* Transform that into a random int from [0, range) */
  result = (int)(random_num * range);
  
  return result;
}

